@@ -11,6 +11,7 @@ impl PdfReaderPosition {
         Self { index: 0 }
     }
 }
+
 pub struct PdfParser<'a> {
     buffer: &'a [u8],
 }
@@ -67,6 +68,8 @@ impl<'a> PdfParser<'a> {
             return Some(PdfObject::Boolean(true));
         } else if next_word == b"false" {
             return Some(PdfObject::Boolean(false));
+        } else if next_word == b"null" {
+            return Some(PdfObject::Null);
         } else if next_word
             .iter()
             .enumerate()
@@ -95,8 +98,17 @@ impl<'a> PdfParser<'a> {
 
             return Some(PdfObject::Integer(first_number));
         } else if next_word == b"/" {
-            let name_word = self.next_word(position)?;
+            let current_byte = self.current_value(position);
+            let name_word = if Self::is_whitespace(current_byte) || Self::is_delimiter(current_byte)
+            {
+                return Some(PdfObject::Name(
+                    &self.buffer[position.index..position.index],
+                ));
+            } else {
+                self.next_word(position)?
+            };
             return if Self::is_delimiter(name_word[0]) {
+                // TODO: Return error?
                 None
             } else {
                 Some(PdfObject::Name(name_word))
@@ -175,7 +187,7 @@ impl<'a> PdfParser<'a> {
             if !self.advance_position(position) {
                 return None;
             }
-            return Some(PdfObject::String(PdfString::new(slice)));
+            return Some(PdfObject::String(PdfString::new_literal(slice)));
         } else if next_word == b"<" {
             // Start of hexadecimal string.
             // TODO: Decode (on demand?)
@@ -189,7 +201,7 @@ impl<'a> PdfParser<'a> {
             if !self.advance_position(position) {
                 return None;
             }
-            return Some(PdfObject::String(PdfString::new(slice)));
+            return Some(PdfObject::String(PdfString::new_hexadecimal(slice)));
         }
 
         if let Ok(float_value) = std::str::from_utf8(next_word).ok()?.parse::<f32>() {
@@ -229,7 +241,7 @@ impl<'a> PdfParser<'a> {
     /// Check if a byte represents a whitespace according to the PDF file format.
     ///
     /// See PDF 32000-1:2008, 7.2.2 Character Set.
-    const fn is_whitespace(byte: u8) -> bool {
+    pub const fn is_whitespace(byte: u8) -> bool {
         matches!(byte, 0 | 9 | 10 | 12 | 13 | 32)
     }
 
@@ -315,12 +327,15 @@ impl<'a> PdfParser<'a> {
 #[cfg(test)]
 mod tests {
     use crate::parse::{PdfParser, PdfReaderPosition};
-    use crate::{PdfDocument, PdfObject, PdfObjectIdentifier, PdfString, PdfVersion};
+    use crate::{
+        PdfDocument, PdfDocumentData, PdfFormField, PdfObject, PdfObjectIdentifier, PdfString,
+        PdfVersion,
+    };
 
     #[test]
     fn parse_junk() {
         let pdf_bytes = b"hello world";
-        let parse_result = PdfDocument::parse(pdf_bytes);
+        let parse_result = PdfDocumentData::parse(pdf_bytes);
         assert_eq!(
             parse_result.err().unwrap(),
             "File not starting with '%PDF-'".to_string()
@@ -329,7 +344,7 @@ mod tests {
 
     #[test]
     fn parse_minimal() {
-        fn assert_minimal_first_object(pdf: &PdfDocument) {
+        fn assert_minimal_first_object(pdf: &PdfDocumentData) {
             if let Some(PdfObject::Dictionary(dict)) =
                 pdf.objects.get(&PdfObjectIdentifier::new(1, 0))
             {
@@ -347,7 +362,7 @@ mod tests {
             }
         }
 
-        fn assert_minimal_second_object(pdf: &PdfDocument) {
+        fn assert_minimal_second_object(pdf: &PdfDocumentData) {
             if let Some(PdfObject::Dictionary(dict)) =
                 pdf.objects.get(&PdfObjectIdentifier::new(2, 0))
             {
@@ -374,7 +389,7 @@ mod tests {
             }
         }
 
-        fn assert_minimal_third_object(pdf: &PdfDocument) {
+        fn assert_minimal_third_object(pdf: &PdfDocumentData) {
             if let Some(PdfObject::Dictionary(dict)) =
                 pdf.objects.get(&PdfObjectIdentifier::new(3, 0))
             {
@@ -424,7 +439,7 @@ mod tests {
             }
         }
 
-        fn assert_minimal_fourth_object(pdf: &PdfDocument, unix_line_endings: bool) {
+        fn assert_minimal_fourth_object(pdf: &PdfDocumentData, unix_line_endings: bool) {
             if let Some(PdfObject::Stream(stream)) =
                 pdf.objects.get(&PdfObjectIdentifier::new(4, 0))
             {
@@ -459,7 +474,7 @@ mod tests {
             // Minimal PDF file explained at https://brendanzagaeski.appspot.com/0004.html
             assert!(pdf_bytes.starts_with(b"%PDF-1.1"));
 
-            let pdf = PdfDocument::parse(pdf_bytes).unwrap();
+            let pdf = PdfDocumentData::parse(pdf_bytes).unwrap();
             assert_eq!(pdf.version, PdfVersion::Version11);
             assert_eq!(pdf.objects.len(), 4);
 
@@ -485,9 +500,19 @@ mod tests {
     #[test]
     fn parse_signed_signicat() {
         let pdf_bytes = include_bytes!("tests/assets/signed-by-signicat-example.pdf");
-        let pdf = PdfDocument::parse(pdf_bytes).unwrap();
+        let pdf = PdfDocumentData::parse(pdf_bytes).unwrap();
         assert_eq!(pdf.version, PdfVersion::Version17);
         assert_eq!(pdf.objects.len(), 110);
+
+        let pdf = PdfDocument::parse(pdf_bytes).unwrap();
+        let pdf_fields = pdf.catalog.interactive_form.unwrap().fields;
+        assert_eq!(pdf_fields.len(), 1);
+        if let PdfFormField::Signature(signature_field) = &pdf_fields[0] {
+            if let Some(_signature) = &signature_field.signature {
+                return;
+            }
+        }
+        panic!("Failed parsing signature");
     }
 
     #[test]
@@ -685,7 +710,7 @@ mod tests {
             );
             assert_eq!(
                 dictionary.map.get(&b"T"[..]),
-                Some(&PdfObject::String(PdfString::new(b"Signature1")))
+                Some(&PdfObject::String(PdfString::new_literal(b"Signature1")))
             );
             assert_eq!(
                 dictionary.map.get(&b"V"[..]),
@@ -710,7 +735,7 @@ mod tests {
             assert_eq!(
                 dictionary.map.get(&b"Contents"[..]),
                 // TODO:
-                Some(&PdfObject::String(PdfString::new(b"ff")))
+                Some(&PdfObject::String(PdfString::new_hexadecimal(b"ff")))
             );
             assert_eq!(
                 dictionary.map.get(&b"Reference"[..]),
@@ -721,6 +746,63 @@ mod tests {
         }
     }
 
+    #[test]
+    fn test_parse_dictionary_mapping_to_empty_name() {
+        for bytes in [
+            &b"<< /App << /Name / >> >> "[..],
+            &b"<</App<</Name/>>>> "[..],
+        ] {
+            let (parser, mut position) = (PdfParser::new(bytes), PdfReaderPosition::new());
+            if let Some(PdfObject::Dictionary(dictionary)) = parser.next_object(&mut position) {
+                assert_eq!(dictionary.map.len(), 1);
+                if let Some(PdfObject::Dictionary(dictionary)) = dictionary.map.get(&b"App"[..]) {
+                    assert_eq!(dictionary.map.len(), 1);
+                    assert_eq!(
+                        dictionary.map.get(&b"Name"[..]),
+                        Some(&PdfObject::Name(b""))
+                    );
+                    return;
+                }
+            }
+            panic!("Failed to parse dict");
+        }
+    }
+
+    /// Test that "A dictionary entry whose value is null (see 7.3.9, 'Null object') shall be
+    /// treated the same as if the entry does not exist".
+    ///
+    /// From 7.3.7 Dictionary objects:
+    /// <https://www.adobe.com/content/dam/acom/en/devnet/pdf/pdfs/PDF32000_2008.pdf#page=26>
+    ///
+    /// TODO: Ensure this behaviour also when value is indirect reference to non-existing object.
+    /// TODO: Perhaps by having this logic on a dictionary lookup method?
+    #[test]
+    #[ignore]
+    fn test_parse_dictionary_with_null_value() {
+        let bytes = b"<< /FirstKey 1 /SecondKey null /ThirdKey 3 >> ";
+        let (parser, mut position) = (PdfParser::new(bytes), PdfReaderPosition::new());
+        if let Some(PdfObject::Dictionary(dictionary)) = parser.next_object(&mut position) {
+            assert_eq!(dictionary.map.len(), 2);
+            assert_eq!(
+                dictionary.map.get(&b"FirstKey"[..]),
+                Some(&PdfObject::Integer(1))
+            );
+            assert_eq!(
+                dictionary.map.get(&b"SecondKey"[..]),
+                Some(&PdfObject::Integer(3))
+            );
+            return;
+        }
+        panic!("Failed to parse dict");
+    }
+
+    #[test]
+    fn test_parse_null() {
+        let bytes = b"null ";
+        let (parser, mut position) = (PdfParser::new(bytes), PdfReaderPosition::new());
+        assert_eq!(parser.next_object(&mut position), Some(PdfObject::Null));
+    }
+
     #[test]
     fn test_parse_names() {
         fn assert_parsing(from: &[u8], expected_name: &[u8]) {
@@ -755,22 +837,22 @@ mod tests {
         let (parser, mut position) = (PdfParser::new(bytes), PdfReaderPosition::new());
         assert_eq!(
             parser.next_object(&mut position),
-            Some(PdfObject::String(PdfString::new(b"")))
+            Some(PdfObject::String(PdfString::new_literal(b"")))
         );
 
         let bytes = b"(hello, world) (second string)(third) ";
         let (parser, mut position) = (PdfParser::new(bytes), PdfReaderPosition::new());
         assert_eq!(
             parser.next_object(&mut position),
-            Some(PdfObject::String(PdfString::new(b"hello, world")))
+            Some(PdfObject::String(PdfString::new_literal(b"hello, world")))
         );
         assert_eq!(
             parser.next_object(&mut position),
-            Some(PdfObject::String(PdfString::new(b"second string")))
+            Some(PdfObject::String(PdfString::new_literal(b"second string")))
         );
         assert_eq!(
             parser.next_object(&mut position),
-            Some(PdfObject::String(PdfString::new(b"third")))
+            Some(PdfObject::String(PdfString::new_literal(b"third")))
         );
 
         let bytes = b"(hello \\(world\\) bye) ";
@@ -778,7 +860,9 @@ mod tests {
         assert_eq!(
             parser.next_object(&mut position),
             // TODO: Unescape backslash (on demand?)
-            Some(PdfObject::String(PdfString::new(b"hello \\(world\\) bye")))
+            Some(PdfObject::String(PdfString::new_literal(
+                b"hello \\(world\\) bye"
+            )))
         );
     }
 
@@ -791,11 +875,22 @@ mod tests {
             assert_eq!(array[0], PdfObject::Integer(549));
             assert_eq!(array[1], PdfObject::Real(6.14));
             assert_eq!(array[2], PdfObject::Boolean(false));
-            assert_eq!(array[3], PdfObject::String(PdfString::new(b"Ralph")));
+            assert_eq!(
+                array[3],
+                PdfObject::String(PdfString::new_literal(b"Ralph"))
+            );
             assert_eq!(array[4], PdfObject::Name(b"SomeName"));
         } else {
             panic!("Failed to parse array");
         }
+
+        let bytes = b"[] ";
+        let (parser, mut position) = (PdfParser::new(bytes), PdfReaderPosition::new());
+        if let Some(PdfObject::Array(array)) = parser.next_object(&mut position) {
+            assert_eq!(array.len(), 0);
+        } else {
+            panic!("Failed to parse empty array");
+        }
     }
 
     #[test]
